@@ -10,6 +10,7 @@ from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument
 from hypothesis.strategies._internal.utils import cacheable
 
+from .. import nodes
 from ..types import AstPrinter, CustomScalars, Field, InputTypeNode, InterfaceOrObject, SelectionNodes
 from . import factories, primitives
 from .ast import make_mutation, make_query
@@ -17,6 +18,8 @@ from .containers import flatten
 from .validation import maybe_parse_schema, validate_custom_scalars, validate_fields
 
 BY_NAME = operator.attrgetter("name")
+EMPTY_LISTS_STRATEGY = st.builds(list)
+BUILT_IN_SCALAR_TYPE_NAMES = {"Int", "Float", "String", "ID", "Boolean"}
 
 
 def instance_cache(key_func: Callable) -> Callable:
@@ -47,7 +50,9 @@ class GraphQLStrategy:
     # This is a per-method cache without limits as they are proportionate to the schema size
     _cache: Dict[str, Dict] = attr.ib(factory=dict)
 
-    def values(self, type_: graphql.GraphQLInputType) -> st.SearchStrategy[InputTypeNode]:
+    def values(
+        self, type_: graphql.GraphQLInputType, default: Optional[graphql.ValueNode] = None
+    ) -> st.SearchStrategy[InputTypeNode]:
         """Generate value nodes of a type, that corresponds to the input type.
 
         They correspond to all `GraphQLInputType` variants:
@@ -64,32 +69,40 @@ class GraphQLStrategy:
         if isinstance(type_, graphql.GraphQLScalarType):
             type_name = type_.name
             if type_name in self.custom_scalars:
-                return primitives.maybe_null(self.custom_scalars[type_name], nullable)
-            return primitives.scalar(type_name, nullable)
+                return primitives.custom(self.custom_scalars[type_name], nullable, default=default)
+            return primitives.scalar(type_name, nullable, default=default)
         if isinstance(type_, graphql.GraphQLEnumType):
             values = tuple(type_.values)
-            return primitives.enum(values, nullable)
+            return primitives.enum(values, nullable, default=default)
         # Types with children
         if isinstance(type_, graphql.GraphQLList):
-            return self.lists(type_, nullable)
+            return self.lists(type_, nullable, default=default)
         if isinstance(type_, graphql.GraphQLInputObjectType):
             return self.objects(type_, nullable)
         raise TypeError(f"Type {type_.__class__.__name__} is not supported.")
 
-    @instance_cache(lambda type_, nullable=True: (make_type_name(type_), nullable))
-    def lists(self, type_: graphql.GraphQLList, nullable: bool = True) -> st.SearchStrategy[graphql.ListValueNode]:
+    @instance_cache(lambda type_, nullable=True, default=None: (make_type_name(type_), nullable, default))
+    def lists(
+        self, type_: graphql.GraphQLList, nullable: bool = True, default: Optional[graphql.ValueNode] = None
+    ) -> st.SearchStrategy[graphql.ListValueNode]:
         """Generate a `graphql.ListValueNode`."""
         strategy = st.lists(self.values(type_.of_type))
-        return primitives.maybe_null(strategy.map(factories.list_value), nullable)
+        return primitives.list_(strategy, nullable, default=default)
 
     @instance_cache(lambda type_, nullable=True: (type_.name, nullable))
     def objects(
         self, type_: graphql.GraphQLInputObjectType, nullable: bool = True
     ) -> st.SearchStrategy[graphql.ObjectValueNode]:
         """Generate a `graphql.ObjectValueNode`."""
-        fields = {name: field for name, field in type_.fields.items() if self.can_generate_field(field)}
+        fields = {
+            name: field
+            for name, field in type_.fields.items()
+            # Generate optional fields that are possible to generate and all required fields.
+            # If a required field is not possible to generate, then it will fail deeper anyway
+            if self.can_generate_field(field) or graphql.is_required_input_field(field)
+        }
         strategy = subset_of_fields(fields, force_required=True).flatmap(self.lists_of_object_fields)
-        return primitives.maybe_null(strategy.map(factories.object_value), nullable)
+        return primitives.maybe_null(strategy.map(nodes.Object), nullable)
 
     def can_generate_field(self, field: graphql.GraphQLInputField) -> bool:
         """Whether it is possible to generate values for the given field."""
@@ -98,17 +111,22 @@ class GraphQLStrategy:
             # Can generate any non-scalar
             not isinstance(type_, graphql.GraphQLScalarType)
             # Default scalars
-            or type_.name in {"Int", "Float", "String", "ID", "Boolean"}
+            or type_.name in BUILT_IN_SCALAR_TYPE_NAMES
             # User-provided scalars
             or type_.name in self.custom_scalars
         )
 
     def lists_of_object_fields(
-        self, items: List[Tuple[str, Field]]
+        self, items: List[Tuple[str, graphql.GraphQLInputField]]
     ) -> st.SearchStrategy[List[graphql.ObjectFieldNode]]:
-        return st.tuples(*(self.values(field.type).map(factories.object_field(name)) for name, field in items)).map(
-            list
-        )
+        return st.tuples(
+            *(
+                self.values(field.type, field.ast_node.default_value if field.ast_node is not None else None).map(
+                    factories.object_field(name)
+                )
+                for name, field in items
+            )
+        ).map(list)
 
     @instance_cache(lambda interface, implementations: (interface.name, tuple(impl.name for impl in implementations)))
     def interfaces(
@@ -130,7 +148,7 @@ class GraphQLStrategy:
         # then the resulting query should not have these fields simultaneously
         strategies, overlapping_fields = self.collect_fragment_strategies(items)
         if overlapping_fields:
-            return compose_interfaces_with_filter(st.just([]), strategies, self.schema.type_map)
+            return compose_interfaces_with_filter(EMPTY_LISTS_STRATEGY, strategies, self.schema.type_map)
         # No overlapping - safe to choose any subset of fields within the interface itself and any fragment
         return st.tuples(*(self.inline_fragment(type_) for type_ in items)).map(list)
 
@@ -193,17 +211,14 @@ class GraphQLStrategy:
         def inner(draw: Any) -> List[graphql.ArgumentNode]:
             args = []
             for name, argument in arguments.items():
+                default = argument.ast_node.default_value if argument.ast_node is not None else None
                 try:
-                    argument_strategy = self.values(argument.type)
+                    argument_strategy = self.values(argument.type, default=default)
                 except InvalidArgument:
                     if not isinstance(argument.type, graphql.GraphQLNonNull):
                         # If the type is nullable, then either generate `null` or skip it completely
                         if draw(st.booleans()):
-                            args.append(
-                                graphql.ArgumentNode(
-                                    name=graphql.NameNode(value=name), value=primitives.NULL_VALUE_NODE
-                                )
-                            )
+                            args.append(graphql.ArgumentNode(name=graphql.NameNode(value=name), value=nodes.Null))
                         continue
                     raise
                 args.append(draw(argument_strategy.map(factories.argument(name))))
@@ -300,8 +315,8 @@ def compose_interfaces_with_filter(
 
 
 def subset_of_fields(
-    fields: Dict[str, Field], *, force_required: bool = False
-) -> st.SearchStrategy[List[Tuple[str, Field]]]:
+    fields: Dict[str, graphql.GraphQLInputField], *, force_required: bool = False
+) -> st.SearchStrategy[List[Tuple[str, graphql.GraphQLInputField]]]:
     """A helper to select a subset of fields."""
     field_pairs = sorted(fields.items())
     # if we need to always generate required fields, then return them and extend with a subset of optional fields
