@@ -1,6 +1,7 @@
-from typing import List, Union
+from typing import Callable, Dict, List, Union
 
 import graphql
+from hypothesis import strategies as st
 
 # Leaf nodes, that don't have children
 ScalarValueNode = Union[graphql.IntValueNode, graphql.FloatValueNode, graphql.StringValueNode, graphql.BooleanValueNode]
@@ -8,3 +9,5 @@ InputTypeNode = Union[ScalarValueNode, graphql.EnumValueNode, graphql.ListValueN
 Field = Union[graphql.GraphQLField, graphql.GraphQLInputField]
 InterfaceOrObject = Union[graphql.GraphQLObjectType, graphql.GraphQLInterfaceType]
 SelectionNodes = List[graphql.SelectionNode]
+AstPrinter = Callable[[graphql.Node], str]
+CustomScalars = Dict[str, st.SearchStrategy[graphql.ValueNode]]
